const htmlparser = require('htmlparser2');

function attribsReplace(originTag, weexTag, attrName, attrValue) {
  if (attrValue === undefined) return attrName;

  let quot = attrValue.indexOf('"') > -1 ? '\'' : '"';

  attrName = (attrName || '');

  switch (attrName.toLowerCase()) {
    default: {
      let m = attrName.match(/^v-tap(\.stop|prevent)?/);
      if (m) {
        //处理 v-tap
        //这个正则匹配：someMethods.bind(this, 'abc', 123)
        let regBindThis = /(\w+)\.bind\s*\(\s*this\s*,\s*/gmi;

        attrValue = attrValue.replace(regBindThis, '$1(');

        return ` @click=${quot}${attrValue}${quot}`;
      }
    }
  }

  return ` ${attrName}=${quot}${attrValue}${quot}`
}

function compile(content) {

  let promise = new Promise((resolve) => {
    //最后的总输出用数组存下来
    let output = [];
    //用入栈出栈的形式来准确处理 closetag
    let tagStack = [];
    let parser = new htmlparser.Parser({
      onopentag: function (name, attribs) {

        let attribsStr = '';
        for (let attrName in attribs) {
          attribsStr += attribsReplace(name, name, attrName, attribs[attrName]);
        }

        let tagContent = `<${name}${attribsStr}>`;

        tagStack.push({
          tagname: name,
          attribs: attribs
        });

        output.push(tagContent);
      },
      ontext: function (text) {

        output.push(text);
      },
      onclosetag: function () {
        let lastTag = tagStack.pop();
        output.push(`</${lastTag.tagname}>`);
      },
      onend: function () {
        let template = output.join('');
        resolve(template);
      }
    }, {
      recognizeSelfClosing: true,
      lowerCaseAttributeNames: false,
      lowerCaseTags: false,
      decodeEntities: true
    });

    parser.write(content);
    parser.end();
  });

  return promise;
}

module.exports = compile;
