const utils = require('../build/utils');
const path = require('path');
const read = require('fs-readdir-recursive');
const entryFile = path.resolve(__dirname, 'page-entry.js');

function resolve(dir) {
  return path.join(utils.source(), dir)
}

/**
 * 遍历整个 src/pages 目录，查找 .vue 文件
 * @param  {Boolean} isDev [description]
 * @return {[type]}        [description]
 */
function getVuePages(isDev) {

  let sourcePath = utils.source();
  let pagesPath = path.resolve(sourcePath, 'src/pages');

  let list = read(pagesPath); //不要用官方filter，windows下有bug

  //统一 window mac 的路径风格
  list = list.map(item => {
    return item.split(path.sep).join('/');
  });

  list = list.filter(item => {
    let flag = /\.vue$/.test(item); //获取.vue结尾文件
    if (!isDev) {
      flag = flag && !/^demo\//.test(item); //过滤构建稿目录（demo）
    }

    return flag;
  });

  let routes = list.map(item => {
    let page = item.replace(/\.vue$/, '');
    return {
      file: path.resolve(sourcePath, 'src/pages', item),
      page: page
    };
  });

  return routes;
}

/**
 * 生成 mpvue-loader 需要的 entry 节点
 * @return {[type]} [description]
 */
exports.getEntry = function () {
  let vuePages = getVuePages();

  let pagesEntry = vuePages.reduce((res, item) => {

    res['pages/' + item.page] = entryFile + '?page=' + item.file;

    return res;
  }, {});

  let appEntry = {
    app: resolve('./src/main.js')
  };

  let entry = Object.assign({}, appEntry, pagesEntry);

  return entry;
};

exports.getLoader = function () {
  return {
    test: function (file) {
      return file === entryFile;
      // return AllVueFiles[file] !== undefined;
      // return file === entryFile;
    },
    use: [
      {
        loader: path.resolve(__dirname, 'page-entry-loader.js')
      }
    ]
  }
};