const {parseConfig} = require('mpvue-loader/lib/mp-compiler/parse');

const {
  genScript,
  genStyle,
  genPageWxml
} = require('mpvue-loader/lib/mp-compiler/templates');

const {
  getFileInfo,
  getCompNameBySrc,
  resolveTarget,
  getBabelrc,
  cacheFileInfo
} = require('mpvue-loader/lib/mp-compiler/util');

const url = require('url');
const querystring = require('querystring');
const babel = require('babel-core')

function findEntryKey(entry, page) {
  for (let key in entry) {
    let entryItem = url.parse(entry[key]);
    let entryQuery = querystring.parse(entryItem.query);
    if (entryQuery.page === page) {
      return key;
    }
  }
}

module.exports = function () {
  let resource = url.parse(this.resource)
  let query = querystring.parse(resource.query)

  const {emitError, emitFile, emitWarning, resolve, context, options} = this

  const entryContent = `
    import Vue from 'vue'
    import App from '${query.page}'
    
    const app = new Vue(App)
    app.$mount()
  `;

  const babelrc = getBabelrc()
  const {metadata} = babel.transform(entryContent, {
    extends: babelrc,
    plugins: [parseConfig]
  });

  // metadata: config
  const {rootComponent} = metadata

  const pageFile = query.page;

  const entryKey = findEntryKey(options.entry, pageFile);
  const fakeEntry = {};
  fakeEntry[entryKey] = pageFile;
  const fileInfo = resolveTarget(pageFile, fakeEntry)
  cacheFileInfo(pageFile, fileInfo)

  const {src, name, isApp, isPage} = fileInfo

  if (isApp || isPage) {
    // 生成入口 js
    emitFile(`${src}.js`, genScript(name, isPage, src))

    // 生成入口 wxss
    emitFile(`${src}.wxss`, genStyle(name, isPage, src))

    // 这儿应该异步在所有的模块都清晰后再生成
    // 生成入口 wxml
    if (isPage && rootComponent) {
      resolve(context, rootComponent, (err, rootComponentSrc) => {
        if (err) return
        // 这儿需要搞定 根组件的 路径
        createWxml(emitWarning, emitError, emitFile, pageFile, rootComponentSrc)
      })
    }
  }

  return entryContent
};

function createWxml(emitWarning, emitError, emitFile, resourcePath, rootComponent, compiled, html) {
  const {pageType, components, src} = getFileInfo(resourcePath) || {}

  // 这儿一个黑魔法，和 webpack 约定的规范写法有点偏差！
  if (!pageType || (components && !components.isCompleted)) {
    return setTimeout(createWxml, 20, ...arguments)
  }

  let wxmlContent = '';
  let wxmlSrc = '';

  if (rootComponent) {
    const componentName = getCompNameBySrc(rootComponent)
    wxmlContent = genPageWxml(componentName, src)
    wxmlSrc = src
  }

  emitFile(`${wxmlSrc}.wxml`, wxmlContent)
}